import cf
import tempfile
import os
import sys
import numpy

'''
Test the cf package:

TEST: Create a space in memory
TEST: Print a dump of the space
TEST: Write the space to disk
TEST: Read the space from disk, with data stored as netCDF4.Variable objects
TEST: Access the space's data as a numpy array
TEST: Access the space's coordinates' data as a numpy arrays
TEST: Slice the space
TEST: Check the equality function
TEST: Create list of spaces
TEST: Write a list of spaces to disk
TEST: Read a list of spaces from disk
TEST: Print all last spaces in the list
TEST: Print the last space in the list
TEST: Print the data of the last space in the list
TEST: Modify the last space in the list
'''

print '\nTEST: Create a space in memory:'

s = cf.Space()
s.grid = cf.Grid()

s.grid['dim0'] = cf.Coordinate()
s.grid['dim0'].standard_name = 'latitude'
s.grid['dim0']._data = numpy.arange(10, dtype = float)

s.grid['dim1'] = cf.Coordinate()
s.grid['dim1'].standard_name = 'longitude'
s.grid['dim1']._data = numpy.arange(9, dtype = float) + 20

s.grid.dimensions = {'data': ['dim0', 'dim1'],
                     'dim0': ['dim0'],
                     'dim1': ['dim1'],
                     }

s.grid.dimension_sizes = {'dim0': 10,
                          'dim1': 9,
                          }

s.standard_name = 'test_space'
s._data = numpy.arange(90, dtype = float).reshape(10,9)

print '\nTEST: Print a dump of the space:'
try:
    print repr(s)
except:
    raise RuntimeError('Failed to repr the space')
try:
    cf.dump(s)
except:
    raise RuntimeError('Failed to dump the space')

print '\nTEST: Write the space to disk:'
file = tempfile.mktemp('.nc')
try:
    cf.write(s, file)
except:
    raise RuntimeError('Failed to write the space to disk')

print '\nTEST: Read the space from disk, with data stored as netCDF4.Variable objects: '
try:
    t = cf.read1(file)
except:
    raise RuntimeError('Failed to read the space from disk')

try:
    cf.dump(t)
except:
    raise RuntimeError('Failed to dump the read-in space')

print '\nTEST: Access the space\'s data as a numpy array:'
try:
    print t.varray
except:
    raise RuntimeError('Failed to print the space\'s data')

print '\nTEST: Access the space\'s coordinates\' data as a numpy arrays:'
try:
    print t.coord('lat').varray
    print t.coord('lon').varray
except:
    raise RuntimeError('Failed to print the space\'s coordinates\' data')

print '\nTEST: Slice the space:'
try:
    print t(lat = cf.lt(5), lon = 27)
except:
    raise RuntimeError('Failed to slice the space')

print '\nTEST: Slice the space:'
try:
    print t[...,2:5]
except:
    raise RuntimeError('Failed to slice the space')

print '\nTEST: Check the equality function:'
try:
    result  = cf.equals(t, t)
except:
    raise RuntimeError('Failed to check the space is equal to itself')
if not result:
    raise RuntimeError('Space is not equal to itself')

print '\nTEST: Create list of spaces:'
try:
    sl = cf.SpaceList([t,t,t,t])
except:
    raise RuntimeError('Failed to Create list of spaces')

print '\nTEST: Write a list of spaces to disk:'
try:
    cf.write(sl, file)
except:
    raise RuntimeError('Failed to Write a list of spaces to disk')

print '\nTEST: Read a list of spaces from disk:'
try:
    sl = cf.read(file)
    print repr(sl)
except:
    raise RuntimeError('Failed to Read a list of spaces from disk')

print '\nTEST: Print all last spaces in the list:'
try:
    print sl
except:
    raise RuntimeError('Failed to Print all spaces in the list')

print '\nTEST: Print the last space in the list:'
try:
    print sl[-1]
except:
    raise RuntimeError('Failed to Print the last space in the list')

print '\nTEST: Print the data of the last space in the list:'
try:
    print sl[-1].varray
except:
    raise RuntimeError('Failed to Print the data of the last space in the list')

print '\nTEST: Modify the last space in the list:'
try:
    sl[-1] *= -1
    print sl[-1].varray
except:
    raise RuntimeError('Failed to Modify the last space in the list')

print '\n-------------------------------------------'
print   'TEST: All tests passed for cf version',cf.__version__
print   '------------------------------------------- '

# Tidy up
os.remove(file)

